{#
    basic/localtoc.html
    ~~~~~~~~~~~~~~~~~~~

    Sphinx sidebar template: local table of contents.

    :copyright: Copyright 2007-2017 by the Sphinx team, see AUTHORS.
    :license: BSD, see LICENSE for details.

    Modified to work properly with the index page
#}
{%- if display_toc %}
<h3><a href="{{ pathto(master_doc) }}">{{ _('Table Of Contents') }}</a></h3>
{{ toc }}
{%- endif %}
